#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/wait.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <regex.h>

#define STATE_OK 0
#define STATE_CRITICAL 2

static
void ok(void)
{
	printf("OK - process is running\n");
	exit(STATE_OK);
}

static
void crit(const char *msg)
{
	printf("CRITICAL - %s\n", msg);
	exit(STATE_CRITICAL);
}

static
void alarm_handler(int sig)
{
	kill(SIGKILL, 0);
	crit("timeout");
}

static
void trim(char *s) {
	char *p = s;
	int l = strlen(p);

	while (isspace(p[l-1]))
		p[--l] = '\0';

	while (*p && isspace(*p))
		++p, --l;

	memmove(s, p, l+1);
}

int main(int argc, char *argv[])
{
	setresuid(0, 0, 0);

	signal(SIGALRM, alarm_handler);
	alarm(10);

	char *pidfile = NULL;
	char *pregex = NULL;

	// parse arguments
	if (argc < 2)
		crit("no pidfile given");
	else
		pidfile = argv[1];

	if (argc > 2)
		pregex = argv[2];

	// open pidfile
	FILE *fp = fopen(pidfile, "r");

	if (fp == NULL)
		crit("pidfile does not exist");

	// read and trim pid
	char *pidstr = NULL;
	size_t len = 0;

	if (getline(&pidstr, &len, fp) == -1)
		crit("cannot read pidfile");

	fclose(fp);

	trim(pidstr);

	// check if /proc/<pid> exists
	char *cmdlinefile = NULL;
	if (asprintf(&cmdlinefile, "/proc/%s/cmdline", pidstr) == -1)
		crit("no memory");

	struct stat sb;
	if (stat(cmdlinefile, &sb) == -1 || !S_ISREG(sb.st_mode))
		crit("no process found");

	// check if process matches pregex
	if (pregex == NULL)
		ok();

	fp = fopen(cmdlinefile, "r");

	if (fp == NULL)
		crit("pidfile does not exist");

	char *cmdline = NULL;
	len = 0;

	if (getline(&cmdline, &len, fp) == -1)
		crit("cannot read cmdline");

	fclose(fp);

	regex_t re;
	if (regcomp(&re, pregex, REG_EXTENDED) != 0)
		crit("failed to compile regex");

	if (regexec(&re, cmdline, 0, NULL, 0) == 0)
		ok();

	crit("process does not match regex");
	return STATE_CRITICAL;
}
